@@ -98,6 +98,10 @@ impl<K: Hash> Hash for KeyRef<K> {
 }
 
 impl<K: PartialEq> PartialEq for KeyRef<K> {
+    // NB: The unconditional_recursion lint was added in 1.76.0 and can be removed
+    // once the current stable version of Rust is 1.76.0 or higher.
+    #![allow(unknown_lints)]
+    #[allow(clippy::unconditional_recursion)]
     fn eq(&self, other: &KeyRef<K>) -> bool {
         unsafe { (*self.k).eq(&*other.k) }
     }
@@ -124,6 +128,10 @@ impl<K: ?Sized + Hash> Hash for KeyWrapper<K> {
 }
 
 impl<K: ?Sized + PartialEq> PartialEq for KeyWrapper<K> {
+    // NB: The unconditional_recursion lint was added in 1.76.0 and can be removed
+    // once the current stable version of Rust is 1.76.0 or higher.
+    #![allow(unknown_lints)]
+    #[allow(clippy::unconditional_recursion)]
     fn eq(&self, other: &Self) -> bool {
         self.0.eq(&other.0)
     }
@@ -186,6 +194,22 @@ pub struct LruCache<K, V, S = DefaultHasher> {
     tail: *mut LruEntry<K, V>,
 }
 
+impl<K, V> Clone for LruCache<K, V>
+where
+    K: Hash + PartialEq + Eq + Clone,
+    V: Clone,
+{
+    fn clone(&self) -> Self {
+        let mut new_lru = LruCache::new(self.cap());
+
+        for (key, value) in self.iter().rev() {
+            new_lru.push(key.clone(), value.clone());
+        }
+
+        new_lru
+    }
+}
+
 impl<K: Hash + Eq, V> LruCache<K, V> {
     /// Creates a new LRU Cache that holds at most `cap` items.
     ///
@@ -462,6 +486,42 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
         }
     }
 
+    /// Returns a key-value references pair of the key in the cache or `None` if it is not
+    /// present in the cache. Moves the key to the head of the LRU list if it exists.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use lru::LruCache;
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
+    ///
+    /// cache.put(String::from("1"), "a");
+    /// cache.put(String::from("2"), "b");
+    /// cache.put(String::from("2"), "c");
+    /// cache.put(String::from("3"), "d");
+    ///
+    /// assert_eq!(cache.get_key_value("1"), None);
+    /// assert_eq!(cache.get_key_value("2"), Some((&String::from("2"), &"c")));
+    /// assert_eq!(cache.get_key_value("3"), Some((&String::from("3"), &"d")));
+    /// ```
+    pub fn get_key_value<'a, Q>(&'a mut self, k: &Q) -> Option<(&'a K, &'a V)>
+    where
+        K: Borrow<Q>,
+        Q: Hash + Eq + ?Sized,
+    {
+        if let Some(node) = self.map.get_mut(KeyWrapper::from_ref(k)) {
+            let node_ptr: *mut LruEntry<K, V> = node.as_ptr();
+
+            self.detach(node_ptr);
+            self.attach(node_ptr);
+
+            Some(unsafe { (&*(*node_ptr).key.as_ptr(), &*(*node_ptr).val.as_ptr()) })
+        } else {
+            None
+        }
+    }
+
     /// Returns a reference to the value of the key in the cache if it is
     /// present in the cache and moves the key to the head of the LRU list.
     /// If the key does not exist the provided `FnOnce` is used to populate
@@ -531,9 +591,9 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     /// let b = ||->Result<&str, String> {Ok("b")};
     /// assert_eq!(cache.try_get_or_insert(2, a), Ok(&"c"));
     /// assert_eq!(cache.try_get_or_insert(3, a), Ok(&"d"));
-    /// assert_eq!(cache.try_get_or_insert(1, f), Err("failed".to_owned()));
-    /// assert_eq!(cache.try_get_or_insert(1, b), Ok(&"b"));
-    /// assert_eq!(cache.try_get_or_insert(1, a), Ok(&"b"));
+    /// assert_eq!(cache.try_get_or_insert(4, f), Err("failed".to_owned()));
+    /// assert_eq!(cache.try_get_or_insert(5, b), Ok(&"b"));
+    /// assert_eq!(cache.try_get_or_insert(5, a), Ok(&"b"));
     /// ```
     pub fn try_get_or_insert<F, E>(&mut self, k: K, f: F) -> Result<&V, E>
     where
@@ -547,19 +607,15 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
 
             unsafe { Ok(&*(*node_ptr).val.as_ptr()) }
         } else {
-            match f() {
-                Err(e) => Err(e),
-                Ok(v) => {
-                    let (_, node) = self.replace_or_create_node(k, v);
-                    let node_ptr: *mut LruEntry<K, V> = node.as_ptr();
+            let v = f()?;
+            let (_, node) = self.replace_or_create_node(k, v);
+            let node_ptr: *mut LruEntry<K, V> = node.as_ptr();
 
-                    self.attach(node_ptr);
+            self.attach(node_ptr);
 
-                    let keyref = unsafe { (*node_ptr).key.as_ptr() };
-                    self.map.insert(KeyRef { k: keyref }, node);
-                    Ok(unsafe { &*(*node_ptr).val.as_ptr() })
-                }
-            }
+            let keyref = unsafe { (*node_ptr).key.as_ptr() };
+            self.map.insert(KeyRef { k: keyref }, node);
+            Ok(unsafe { &*(*node_ptr).val.as_ptr() })
         }
     }
 
@@ -609,6 +665,58 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
         }
     }
 
+    /// Returns a mutable reference to the value of the key in the cache if it is
+    /// present in the cache and moves the key to the head of the LRU list.
+    /// If the key does not exist the provided `FnOnce` is used to populate
+    /// the list and a mutable reference is returned. If `FnOnce` returns `Err`,
+    /// returns the `Err`.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use lru::LruCache;
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
+    ///
+    /// cache.put(1, "a");
+    /// cache.put(2, "b");
+    /// cache.put(2, "c");
+    ///
+    /// let f = ||->Result<&str, String> {Err("failed".to_owned())};
+    /// let a = ||->Result<&str, String> {Ok("a")};
+    /// let b = ||->Result<&str, String> {Ok("b")};
+    /// if let Ok(v) = cache.try_get_or_insert_mut(2, a) {
+    ///     *v = "d";
+    /// }
+    /// assert_eq!(cache.try_get_or_insert_mut(2, a), Ok(&mut "d"));
+    /// assert_eq!(cache.try_get_or_insert_mut(3, f), Err("failed".to_owned()));
+    /// assert_eq!(cache.try_get_or_insert_mut(4, b), Ok(&mut "b"));
+    /// assert_eq!(cache.try_get_or_insert_mut(4, a), Ok(&mut "b"));
+    /// ```
+    pub fn try_get_or_insert_mut<'a, F, E>(&'a mut self, k: K, f: F) -> Result<&'a mut V, E>
+    where
+        F: FnOnce() -> Result<V, E>,
+    {
+        if let Some(node) = self.map.get_mut(&KeyRef { k: &k }) {
+            let node_ptr: *mut LruEntry<K, V> = node.as_ptr();
+
+            self.detach(node_ptr);
+            self.attach(node_ptr);
+
+            unsafe { Ok(&mut *(*node_ptr).val.as_mut_ptr()) }
+        } else {
+            let v = f()?;
+            let (_, node) = self.replace_or_create_node(k, v);
+            let node_ptr: *mut LruEntry<K, V> = node.as_ptr();
+
+            self.attach(node_ptr);
+
+            let keyref = unsafe { (*node_ptr).key.as_ptr() };
+            self.map.insert(KeyRef { k: keyref }, node);
+            unsafe { Ok(&mut *(*node_ptr).val.as_mut_ptr()) }
+        }
+    }
+
     /// Returns a reference to the value corresponding to the key in the cache or `None` if it is
     /// not present in the cache. Unlike `get`, `peek` does not update the LRU list so the key's
     /// position will be unchanged.
@@ -1155,7 +1263,7 @@ impl<'a, K: Hash + Eq, V, S: BuildHasher> IntoIterator for &'a mut LruCache<K, V
 unsafe impl<K: Send, V: Send, S: Send> Send for LruCache<K, V, S> {}
 unsafe impl<K: Sync, V: Sync, S: Sync> Sync for LruCache<K, V, S> {}
 
-impl<K: Hash + Eq, V> fmt::Debug for LruCache<K, V> {
+impl<K: Hash + Eq, V, S: BuildHasher> fmt::Debug for LruCache<K, V, S> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("LruCache")
             .field("len", &self.len())
@@ -1494,6 +1602,26 @@ mod tests {
         assert_eq!(cache.get_or_insert_mut("lemon", || "red"), &"orange");
     }
 
+    #[test]
+    fn test_try_get_or_insert_mut() {
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
+
+        cache.put(1, "a");
+        cache.put(2, "b");
+        cache.put(2, "c");
+
+        let f = || -> Result<&str, &str> { Err("failed") };
+        let a = || -> Result<&str, &str> { Ok("a") };
+        let b = || -> Result<&str, &str> { Ok("b") };
+        if let Ok(v) = cache.try_get_or_insert_mut(2, a) {
+            *v = "d";
+        }
+        assert_eq!(cache.try_get_or_insert_mut(2, a), Ok(&mut "d"));
+        assert_eq!(cache.try_get_or_insert_mut(3, f), Err("failed"));
+        assert_eq!(cache.try_get_or_insert_mut(4, b), Ok(&mut "b"));
+        assert_eq!(cache.try_get_or_insert_mut(4, a), Ok(&mut "b"));
+    }
+
     #[test]
     fn test_put_and_get_mut() {
         let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
@@ -2139,6 +2267,44 @@ mod tests {
         assert_eq!(cache.pop_lru(), Some((0, 0)));
         assert_eq!(cache.pop_lru(), None);
     }
+
+    #[test]
+    fn test_get_key_value() {
+        use alloc::string::String;
+
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
+
+        let key = String::from("apple");
+        cache.put(key, "red");
+
+        assert_eq!(
+            cache.get_key_value("apple"),
+            Some((&String::from("apple"), &"red"))
+        );
+        assert_eq!(cache.get_key_value("banana"), None);
+    }
+
+    #[test]
+    fn test_clone() {
+        let mut cache = LruCache::new(NonZeroUsize::new(3).unwrap());
+        cache.put("a", 1);
+        cache.put("b", 2);
+        cache.put("c", 3);
+
+        let mut cloned = cache.clone();
+
+        assert_eq!(cache.pop_lru(), Some(("a", 1)));
+        assert_eq!(cloned.pop_lru(), Some(("a", 1)));
+
+        assert_eq!(cache.pop_lru(), Some(("b", 2)));
+        assert_eq!(cloned.pop_lru(), Some(("b", 2)));
+
+        assert_eq!(cache.pop_lru(), Some(("c", 3)));
+        assert_eq!(cloned.pop_lru(), Some(("c", 3)));
+
+        assert_eq!(cache.pop_lru(), None);
+        assert_eq!(cloned.pop_lru(), None);
+    }
 }
 
 /// Doctests for what should *not* compile
